<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Block;

use Exception;
use Magento\Bundle\Api\Data\LinkInterface;
use Magento\Catalog\Block\Product\AbstractProduct;
use Magento\Catalog\Block\Product\Context;
use Magento\Catalog\Model\Product;
use Magento\Customer\Helper\View;
use Magento\Customer\Model\Context as CustomerContext;
use Magento\Customer\Model\SessionFactory;
use Magento\Framework\App\Http\Context as HttpContext;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Message\ManagerInterface;
use Mageplaza\NameYourPrice\Helper\Data as HelperData;
use Mageplaza\NameYourPrice\Model\ConditionFactory;
use Mageplaza\NameYourPrice\Model\RequestsFactory;
use Mageplaza\NameYourPrice\Model\ResourceModel\Requests\Collection;
use Mageplaza\NameYourPrice\Model\ResourceModel\Requests\CollectionFactory;
use Mageplaza\NameYourPrice\Model\ResourceModel\RequestsFactory as ResourceFactory;
use Magento\Bundle\Api\ProductLinkManagementInterface;
use Magento\Catalog\Model\ProductFactory;
use RuntimeException;
use Magento\Framework\App\ResourceConnection;
use Magento\Bundle\Model\Product\Type as Bundle;
use Magento\GroupedProduct\Model\Product\Type\Grouped;

/**
 * Class Bargain
 * @package Mageplaza\NameYourPrice\Block
 */
class Bargain extends AbstractProduct
{
    /**
     * @var HelperData
     */
    protected $_helperData;

    /**
     * @var HttpContext
     */
    protected $_context;

    /**
     * @var SessionFactory
     */
    protected $_sessionFactory;

    /**
     * @var ConditionFactory
     */
    protected $_conditionFactory;

    /**
     * @var RequestsFactory
     */
    protected $_requestFactory;

    /**
     * @var ResourceFactory
     */
    protected $_resourceFactory;

    /**
     * @var CollectionFactory
     */
    protected $_collectionFactory;

    /**
     * @var ManagerInterface
     */
    protected $_messageManager;

    /**
     * @var View
     */
    protected $_customerView;

    /**
     * @var ProductLinkManagementInterface
     */
    protected $productLinkManager;

    /**
     * @var ProductFactory
     */
    protected $productFactory;

    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * Bargain constructor.
     *
     * @param Context $context
     * @param HelperData $helperData
     * @param HttpContext $httpContext
     * @param SessionFactory $sessionFactory
     * @param ConditionFactory $conditionFactory
     * @param RequestsFactory $requestsFactory
     * @param ResourceFactory $resourceFactory
     * @param CollectionFactory $collectionFactory
     * @param ManagerInterface $messageManager
     * @param View $customerView
     * @param ProductLinkManagementInterface $productLinkManager
     * @param ProductFactory $productFactory
     * @param ResourceConnection $resourceConnection
     * @param array $data
     */
    public function __construct(
        Context $context,
        HelperData $helperData,
        HttpContext $httpContext,
        SessionFactory $sessionFactory,
        ConditionFactory $conditionFactory,
        RequestsFactory $requestsFactory,
        ResourceFactory $resourceFactory,
        CollectionFactory $collectionFactory,
        ManagerInterface $messageManager,
        View $customerView,
        ProductLinkManagementInterface $productLinkManager,
        ProductFactory $productFactory,
        ResourceConnection $resourceConnection,
        array $data = []
    ) {
        $this->_helperData        = $helperData;
        $this->_context           = $httpContext;
        $this->_sessionFactory    = $sessionFactory;
        $this->_conditionFactory  = $conditionFactory;
        $this->_requestFactory    = $requestsFactory;
        $this->_resourceFactory   = $resourceFactory;
        $this->_collectionFactory = $collectionFactory;
        $this->_messageManager    = $messageManager;
        $this->_customerView      = $customerView;
        $this->productLinkManager = $productLinkManager;
        $this->productFactory     = $productFactory;
        $this->resourceConnection = $resourceConnection;

        parent::__construct($context, $data);
    }

    /**
     * @param string $fieldName
     *
     * @return bool
     */
    public function checkAdditionalInfo($fieldName)
    {
        $fields = explode(',', $this->_helperData->getAdditional());

        return in_array($fieldName, $fields, true);
    }

    /**
     * @return bool
     */
    public function isLogin()
    {
        return $this->_context->getValue(CustomerContext::CONTEXT_AUTH) && $this->isLoggedIn();
    }

    /**
     * @return bool
     */
    public function isLoggedIn()
    {
        $customer = $this->_sessionFactory->create();

        return $customer->isLoggedIn();
    }

    /**
     * @return HelperData
     */
    public function getHelperData()
    {
        return $this->_helperData;
    }

    /**
     * Auto close status bargain
     */
    public function autoCloseBargain()
    {
        $collections = $this->_collectionFactory->create()
            ->addFieldToFilter('status', HelperData::STATUS_APPROVED)
            ->addFieldToFilter('time_use', ['notnull' => true])
            ->addFieldToFilter('time_use', ['lteq' => date('Y-m-d h:i:s')]);

        if ($collections && $collections->getSize() > 0) {
            /** @var Collection[] $collections */
            foreach ($collections as $collection) {
                $collection->setStatus(HelperData::STATUS_CLOSED)->save();
            }
        }
    }

    /**
     * @return string|null
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function getEmailCustomer()
    {
        if ($this->isLogin()) {
            $customer = $this->_sessionFactory->create();

            return $customer->getCustomerData()->getEmail();
        }

        return null;
    }

    /**
     * @return bool
     */
    public function isBargainProduct()
    {
        return $this->checkProductCondition() && $this->checkCustomerGroups();
    }

    /**
     * @return bool
     */
    public function checkProductCondition()
    {
        $productId  = $this->getProduct()->getId();
        $condition  = $this->_helperData->getConditionConfig();
        $productIds = $this->_conditionFactory->create()->getMatchingProductIds($condition);

        return in_array($productId, $productIds, true);
    }

    /**
     * @return bool
     */
    public function checkCustomerGroups()
    {
        $config        = $this->_helperData->getCustomerGroupsConfig();
        $customerGroup = 0;

        if ($this->_sessionFactory->create()->isLoggedIn()) {
            $customerGroup = $this->_sessionFactory->create()->getCustomer()->getGroupId();
        }

        return in_array($customerGroup, explode(',', $config));
    }

    /**
     * @return array
     */
    public function getAssociatedProducts()
    {
        return $this->getProduct()->getTypeInstance()->getAssociatedProducts($this->getProduct());
    }

    /**
     * @param string $type
     * @param null|string $sku
     *
     * @return array
     * @throws Exception
     */
    public function getChildProductGroupedIds($type, $sku = null)
    {
        $ids = [];
        if ($type === Grouped::TYPE_CODE) {
            foreach ($this->getAssociatedProducts() as $item) {
                $ids[] = $item->getId();
            }
        } elseif ($type === Bundle::TYPE_CODE) {
            foreach ($this->getChildProductBundle($sku) as $item) {
                $ids[] = $item->getEntityId();
            }
        }

        return $ids;
    }

    /**
     * @param string $sku
     *
     * @return LinkInterface[]
     */
    public function getChildProductBundle($sku)
    {
        try {
            $items = $this->productLinkManager->getChildren($sku);
        } catch (Exception $exception) {
            throw new RuntimeException($exception->getMessage());
        }

        return $items;
    }

    /**
     * @param int $id
     *
     * @return Product
     */
    public function loadProduct($id)
    {
        return $this->productFactory->create()->load($id);
    }
}
