<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Block\Customer;

use Magento\Catalog\Block\Product\AbstractProduct;
use Magento\Catalog\Block\Product\Context;
use Magento\Catalog\Model\Product;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Mageplaza\NameYourPrice\Helper\Data as HelperData;
use Mageplaza\NameYourPrice\Model\Requests;
use Mageplaza\NameYourPrice\Model\RequestsFactory;
use Magento\GroupedProduct\Model\Product\Type\Grouped;
use Magento\Bundle\Model\Product\Type as BundleProduct;

/**
 * Class Detail
 * @package Mageplaza\NameYourPrice\Block\Customer
 */
class Detail extends AbstractProduct
{
    /**
     * @var string
     */
    protected $_template = 'customer/detail.phtml';

    /**
     * @var RequestsFactory
     */
    protected $_requestsFactory;

    /**
     * @var HelperData
     */
    protected $_helperData;

    /**
     * @var Configurable
     */
    protected $_productConfigurable;

    /**
     * @var Grouped
     */
    protected $groupedProduct;

    /**
     * @var BundleProduct
     */
    protected $bundleProduct;

    /**
     * Detail constructor.
     *
     * @param Context $context
     * @param RequestsFactory $requestsFactory
     * @param HelperData $helperData
     * @param Configurable $productConfigurable
     * @param Grouped $groupedProduct
     * @param BundleProduct $bundleProduct
     * @param array $data
     */
    public function __construct(
        Context $context,
        RequestsFactory $requestsFactory,
        HelperData $helperData,
        Configurable $productConfigurable,
        Grouped $groupedProduct,
        BundleProduct $bundleProduct,
        array $data = []
    ) {
        $this->_requestsFactory     = $requestsFactory;
        $this->_helperData          = $helperData;
        $this->_productConfigurable = $productConfigurable;
        $this->groupedProduct       = $groupedProduct;
        $this->bundleProduct        = $bundleProduct;

        parent::__construct($context, $data);
    }

    /**
     * @return Requests
     */
    public function getBargainRequest()
    {
        $requestId = $this->getRequestId();

        return $this->_requestsFactory->create()->load($requestId);
    }

    /**
     * @param $productId
     *
     * @return Product
     */
    public function getProductById($productId)
    {
        $parentConfigProductId = $this->_productConfigurable->getParentIdsByChild($productId);
        $parentBundleProductId = $this->bundleProduct->getParentIdsByChild($productId);
        $parentGroupProductId  = $this->groupedProduct->getParentIdsByChild($productId);

        if (isset($parentConfigProductId[0])) {
            $productId = $parentConfigProductId[0];

            return $this->_helperData->getProductById($productId);
        }

        if (isset($parentBundleProductId[0])) {
            $productId = $parentBundleProductId[0];

            return $this->_helperData->getProductById($productId);
        }

        if (isset($parentGroupProductId[0])) {
            $productId = $parentGroupProductId[0];

            return $this->_helperData->getProductById($productId);
        }

        return $this->_helperData->getProductById($productId);
    }
}
