<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Controller\Index;

use Exception;
use Magento\Checkout\Model\Cart as CustomerCart;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Mageplaza\NameYourPrice\Helper\Data;
use Mageplaza\NameYourPrice\Model\ResourceModel\Requests;
use Mageplaza\NameYourPrice\Model\ResourceModel\Requests\CollectionFactory;
use Mageplaza\NameYourPrice\Model\ResourceModel\RequestsFactory as ResourceFactory;
use Magento\Bundle\Model\Product\Type as Bundle;
use Magento\GroupedProduct\Model\Product\Type\Grouped;

/**
 * Class Request
 * @package Mageplaza\NameYourPrice\Controller\Index
 */
class Cancel extends Action
{
    /**
     * @var CollectionFactory
     */
    protected $_collectionFactory;

    /**
     * @var CustomerCart
     */
    private $cart;

    /**
     * @var ResourceFactory
     */
    protected $resourceFactory;

    /**
     * Cancel constructor.
     *
     * @param Context $context
     * @param CustomerCart $cart
     * @param CollectionFactory $collectionFactory
     * @param ResourceFactory $resourceFactory
     */
    public function __construct(
        Context $context,
        CustomerCart $cart,
        CollectionFactory $collectionFactory,
        ResourceFactory $resourceFactory
    ) {
        $this->cart               = $cart;
        $this->_collectionFactory = $collectionFactory;
        $this->resourceFactory    = $resourceFactory;

        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|ResultInterface
     * @throws Exception
     */
    public function execute()
    {
        /** @var Requests $requestFactory */
        $requestId = $this->_request->getParam('request_id');
        $type      = $this->_request->getParam('type');

        $collection = $this->_collectionFactory->create()
            ->addFieldToFilter('request_id', $requestId)
            ->setPageSize(1)
            ->getFirstItem();

        try {
            if ($type === Grouped::TYPE_CODE || $type === Bundle::TYPE_CODE) {
                $requestFactory = $this->resourceFactory->create();
                $requestFactory->massCancel(Data::STATUS_REJECT_BY_CUSTOMER, $requestId);
            } else {
                $collection->setStatus(Data::STATUS_REJECT_BY_CUSTOMER)->save();
            }

            $this->cart->getQuote()->collectTotals();
            $this->cart->save();
        } catch (Exception $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        }

        $redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $redirect->setUrl($this->_redirect->getRefererUrl());

        return $redirect;
    }
}
