<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license sliderConfig is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Model\Api;

use Magento\Framework\Api\Search\SearchCriteriaBuilder;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Mageplaza\NameYourPrice\Api\Data\BargainSearchResultInterface;
use Mageplaza\NameYourPrice\Model\ResourceModel\Requests\Collection;
use Mageplaza\NameYourPrice\Api\BargainManagementInterface;
use Mageplaza\NameYourPrice\Helper\Data as HelperData;
use Mageplaza\NameYourPrice\Helper\Email as HelperEmail;
use Mageplaza\NameYourPrice\Api\Data\BargainSearchResultInterfaceFactory;
use Mageplaza\NameYourPrice\Model\RequestsFactory;
use Mageplaza\NameYourPrice\Model\ResourceModel\Requests\CollectionFactory;
use Mageplaza\NameYourPrice\Controller\Adminhtml\Requests;
use Magento\Framework\Message\ManagerInterface as MessageManagerInterface;
use Magento\Customer\Model\CustomerFactory;
use Mageplaza\NameYourPrice\Api\Data\BargainInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Mageplaza\NameYourPrice\Block\Product\View\Bargain;
use Mageplaza\NameYourPrice\Controller\Index\Request;
use Mageplaza\NameYourPrice\Model\ResourceModel\RequestsFactory as ResourceFactory;
use PHPUnit\Util\Exception;

/**
 * Class BargainManagement
 * @package Mageplaza\NameYourPrice\Model\Api
 */
class BargainManagement implements BargainManagementInterface
{
    /**
     * @var HelperData
     */
    protected $helperData;

    /**
     * @var SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var CollectionProcessorInterface
     */
    protected $collectionProcessor;

    /**
     * @var BargainSearchResultInterfaceFactory
     */
    protected $bargainSearchResultFactory;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var RequestsFactory
     */
    protected $requestsFactory;

    /**
     * @var HelperEmail
     */
    protected $helperEmail;

    /**
     * @var Requests
     */
    protected $requests;

    /**
     * @var MessageManagerInterface
     */
    protected $messageManager;

    /**
     * @var CustomerFactory
     */
    protected $customerFactory;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var Bargain
     */
    protected $blockBargain;

    /**
     * @var Request
     */
    protected $requestBargain;

    /**
     * @var ResourceFactory
     */
    protected $resourceFactory;

    /**
     * BargainManagement constructor.
     *
     * @param HelperData $helperData
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param CollectionProcessorInterface $collectionProcessor
     * @param BargainSearchResultInterfaceFactory $bargainSearchResultFactory
     * @param CollectionFactory $collectionFactory
     * @param RequestsFactory $requestsFactory
     * @param HelperEmail $helperEmail
     * @param Requests $requests
     * @param MessageManagerInterface $messageManager
     * @param CustomerFactory $customerFactory
     * @param ProductRepositoryInterface $productRepository
     * @param Bargain $blockBargain
     * @param Request $requestBargain
     * @param ResourceFactory $resourceFactory
     */
    public function __construct(
        HelperData $helperData,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        CollectionProcessorInterface $collectionProcessor,
        BargainSearchResultInterfaceFactory $bargainSearchResultFactory,
        CollectionFactory $collectionFactory,
        RequestsFactory $requestsFactory,
        HelperEmail $helperEmail,
        Requests $requests,
        MessageManagerInterface $messageManager,
        CustomerFactory $customerFactory,
        ProductRepositoryInterface $productRepository,
        Bargain $blockBargain,
        Request $requestBargain,
        ResourceFactory $resourceFactory
    ) {
        $this->helperData                 = $helperData;
        $this->searchCriteriaBuilder      = $searchCriteriaBuilder;
        $this->collectionProcessor        = $collectionProcessor;
        $this->bargainSearchResultFactory = $bargainSearchResultFactory;
        $this->collectionFactory          = $collectionFactory;
        $this->requestsFactory            = $requestsFactory;
        $this->helperEmail                = $helperEmail;
        $this->requests                   = $requests;
        $this->messageManager             = $messageManager;
        $this->customerFactory            = $customerFactory;
        $this->productRepository          = $productRepository;
        $this->blockBargain               = $blockBargain;
        $this->requestBargain             = $requestBargain;
        $this->resourceFactory            = $resourceFactory;
    }

    /**
     * @inheritdoc
     */
    public function getList(SearchCriteriaInterface $searchCriteria = null)
    {
        $this->checkEnabled();

        /** @var Collection $requestCollection */
        $bargainCollection = $this->collectionFactory->create();

        return $this->getListEntity($searchCriteria, $bargainCollection);
    }

    /**
     * @inheritdoc
     */
    public function approve($id)
    {
        $this->checkEnabled();

        $notEditStatus = [
            HelperData::STATUS_REJECT_BY_CUSTOMER,
            HelperData::STATUS_CLOSED,
            HelperData::STATUS_APPROVED
        ];

        $request = $this->requestsFactory->create()->load($id);

        if (!in_array($request->getStatus(), $notEditStatus, true)) {
            try {
                $request->setStatus(HelperData::STATUS_APPROVED);
                $request->setTimeUse($this->requests->getTimeUse());
                $request->save();

                $this->helperEmail->sendEmail(
                    $this->helperEmail->getApproveTemplate(),
                    $request->getCustomerEmail(),
                    $this->requests->getApproveTemplateParams($request, $request->getAdminMessage())
                );

                return true;

            } catch (\Exception $e) {
                $this->messageManager->addErrorMessage(
                    __(
                        'Cannot approve the bargain request ID %1. It has been approved/rejected, closed or cancelled by customer',
                        $e->getMessage()
                    )
                );
            }
        } else {
            $this->messageManager->addErrorMessage(
                __(
                    'Cannot approve the bargain request ID %1. It has been approved/rejected, closed or cancelled by customer',
                    $request->getId()
                )
            );
        }
    }

    /**
     * @inheritdoc
     */
    public function reject($id)
    {
        $this->checkEnabled();
        $notEditStatus = [HelperData::STATUS_REJECT_BY_CUSTOMER, HelperData::STATUS_CLOSED, HelperData::STATUS_REJECT];
        $request       = $this->requestsFactory->create()->load($id);

        if (in_array($request->getStatus(), $notEditStatus, true)) {
            $this->messageManager->addErrorMessage(
                __(
                    'Cannot reject the bargain request ID %1. It has been approved/rejected, closed or cancelled by customer',
                    $request->getId()
                )
            );
        } else {
            try {
                $request->setStatus(HelperData::STATUS_REJECT)->save();
                $this->helperEmail->sendEmail(
                    $this->helperEmail->getRejectTemplate(),
                    $request->getCustomerEmail(),
                    $this->requests->getApproveTemplateParams($request, $request->getAdminMessage())
                );

                return true;
            } catch (\Exception $e) {
                $this->messageManager->addErrorMessage(
                    __(
                        'Cannot reject the bargain request ID %1. It has been approved/rejected, closed or cancelled by customer',
                        $e->getMessage()
                    )
                );
            }
        }
    }

    /**
     * @inheritdoc
     */
    public function saveGuest(BargainInterface $bargain)
    {
        $this->checkEnabled();
        $this->checkRequired($bargain->getData());
        $this->checkData($bargain);

        return $this->processSave($bargain);
    }

    /**
     * @inheritdoc
     */
    public function getMine($customerId, SearchCriteriaInterface $searchCriteria = null)
    {
        $this->checkEnabled();
        $customer = $this->customerFactory->create()->load($customerId);
        $list     = [];

        /** @var Collection $requestCollection */
        $bargainCollection = $this->collectionFactory->create()
            ->addFieldToFilter('customer_email', $customer->getEmail());

        foreach ($bargainCollection as $bargain) {
            $list[] = $bargain;
        }

        return $list;
    }

    /**
     * @inheritdoc
     */
    public function saveMine($customerId, BargainInterface $bargain)
    {
        $this->checkEnabled();
        $this->checkRequired($bargain->getData(), $customerId);
        $customer = $this->customerFactory->create()->load($customerId);
        $bargain->setCustomerEmail($customer->getEmail());
        $bargain->setCustomerName($customer->getName());
        $this->checkData($bargain);

        return $this->processSave($bargain);
    }

    /**
     * @param string $customerId
     * @param string $id
     *
     * @return bool
     * @throws InputException
     */
    public function cancelMine($customerId, $id)
    {
        $customerEmail = $this->customerFactory->create()->load($customerId)->getEmail();
        $ids           = explode(',', $id);
        $collections   = $this->collectionFactory->create()
            ->addFieldToFilter('request_id', ['in' => $ids])
            ->addFieldToFilter('customer_email', $customerEmail);

        if (count($ids) !== $collections->getSize()) {
            throw new InputException(__('The request does not belong to the customer'));
        }

        try {
            $requestFactory = $this->resourceFactory->create();
            $requestFactory->massCancel(HelperData::STATUS_REJECT_BY_CUSTOMER, $id);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }

        return true;
    }

    /**
     * @param $bargain
     *
     * @return mixed
     * @throws InputException
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function processSave($bargain)
    {
        $data           = $this->requestBargain->getData($bargain->getData());
        $requestFactory = $this->resourceFactory->create();
        $minPrice       = $this->blockBargain->getMinPrice($bargain->getProductId());

        if ($bargain->getBargainPrice() < $minPrice) {
            throw new InputException(__('The Bargain Price should be smaller than %1.', $minPrice));
        }

        if ($requestFactory->checkUnique($data['sku'], $bargain->getCustomerEmail())) {
            throw new InputException(__('This email has bargained for this product.'));
        }

        $request = $this->requestsFactory->create();

        try {
            $request->addData($data)->save();
            $this->requestBargain->sendMail($bargain->getData());

            return $request;
        } catch (Exception $e) {
            throw new NoSuchEntityException($e->getMessage());
        }
    }

    /**
     * @inheritdoc
     */
    public function getListEntity($searchCriteria, $collection)
    {
        /** @var BargainSearchResultInterface $searchResult */
        $searchResult = $this->bargainSearchResultFactory->create();

        if ($searchCriteria === null) {
            $searchCriteria = $this->searchCriteriaBuilder->create();
        } else {
            $this->collectionProcessor->process($searchCriteria, $collection);
        }

        $searchResult->setItems($collection->getItems());
        $searchResult->setTotalCount($collection->getSize());
        $searchResult->setSearchCriteria($searchCriteria);

        return $searchResult;
    }

    /**
     * @throws LocalizedException
     */
    public function checkEnabled()
    {
        if (!$this->helperData->isEnabled()) {
            throw new LocalizedException(__('Module is disabled.'));
        }
    }

    /**
     * @param $data
     * @param null $customerId
     *
     * @throws InputException
     */
    public function checkRequired($data, $customerId = null)
    {
        $required = [
            BargainInterface::PRODUCT_ID,
            BargainInterface::BARGAIN_PRICE,
            BargainInterface::BARGAIN_QTY,
            BargainInterface::CUSTOMER_EMAIL
        ];

        if ($customerId) {
            unset($required[3]);
        }

        foreach ($required as $item) {
            if (!array_key_exists($item, $data)) {
                throw new InputException(__('%1 is required', $item));
            }
        }
    }

    /**
     * @param $data
     *
     * @throws InputException
     * @throws NoSuchEntityException
     */
    public function checkData($data)
    {
        $this->productRepository->getById($data->getProductId());

        if ($data->getPhone() && !is_numeric($data->getPhone())) {
            throw new InputException(__('Please enter a number for the phone.'));
        }

        if (!filter_var($data->getCustomerEmail(), FILTER_VALIDATE_EMAIL)) {
            throw new InputException(__('Email address is invalid.'));
        }

        $minPrice = $this->blockBargain->getMinPrice($data->getProductId());

        if ($data->getBargainPrice() < $minPrice) {
            throw new InputException(__('The bargain price should be greater than %1', $minPrice));
        }
    }
}
