<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Model\Api;

use Magento\Framework\DataObject;
use Mageplaza\NameYourPrice\Api\ConfigManagementInterface;
use Mageplaza\NameYourPrice\Helper\Data as HelperData;
use Magento\Framework\App\RequestInterface;
use Mageplaza\NameYourPrice\Helper\Email as HelperEmail;

/**
 * Class ConfigManagement
 * @package Mageplaza\NameYourPrice\Model\Api
 */
class ConfigManagement implements ConfigManagementInterface
{
    /**
     * @var HelperData
     */
    protected $helperData;

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var HelperEmail
     */
    protected $helperEmail;

    /**
     * ConfigManagement constructor.
     *
     * @param HelperData $helperData
     * @param RequestInterface $request
     * @param HelperEmail $helperEmail
     */
    public function __construct(
        HelperData $helperData,
        RequestInterface $request,
        HelperEmail $helperEmail
    ) {
        $this->helperData  = $helperData;
        $this->request     = $request;
        $this->helperEmail = $helperEmail;
    }

    /**
     * @return DataObject
     */
    public function getConfig()
    {
        $helper      = $this->helperData;
        $helperEmail = $this->helperEmail;
        $storeId     = $this->request->getParam('storeId');

        return new DataObject([
            'general_config' => new DataObject([
                'enabled'             => $helper->isEnabled($storeId),
                'customer_groups'     => $helper->getCustomerGroupsConfig($storeId),
                'condition'           => $helper->getConditionConfig($storeId),
                'price_type'          => $helper->getMinPriceType($storeId),
                'price_value'         => $helper->getMinPriceValue($storeId),
                'is_tax'              => $helper->isApplyTax($storeId),
                'allow_discount'      => $helper->isApplyDiscount($storeId),
                'limit_time'          => $helper->getLimitTime($storeId),
                'bargain_qty'         => $helper->isBargainQty($storeId),
                'apply_custom_option' => $helper->isApplyCustomOption($storeId)
            ]),
            'form_config'    => new DataObject([
                'is_popup'     => $helper->isUsePopup($storeId),
                'fields'       => $helper->getAdditional($storeId),
                'button_label' => $helper->getButtonLabel($storeId)
            ]),
            'email_config'   => new DataObject([
                'enabled'          => $helperEmail->isEnabledEmail($storeId),
                'sender'           => $helperEmail->getSender($storeId),
                'admin_email'      => $helperEmail->getToAdminEmail($storeId)
                    ? implode(',', $helperEmail->getToAdminEmail($storeId)) : '',
                'template_confirm' => $helperEmail->getConfirmTemplate($storeId),
                'template_approve' => $helperEmail->getApproveTemplate($storeId),
                'template_reject'  => $helperEmail->getRejectTemplate($storeId)
            ])
        ]);
    }
}
