<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Plugin\Helper\Catalog\Product;

use Magento\Bundle\Helper\Catalog\Product\Configuration as CoreConfiguration;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product\Configuration\Item\ItemInterface;
use Magento\Checkout\Model\Cart;
use Mageplaza\NameYourPrice\Helper\Data;

/**
 * Class Configuration
 * @package Mageplaza\NameYourPrice\Plugin\Helper\Catalog\Product
 */
class Configuration
{
    /**
     * @var Cart
     */
    protected $cart;

    /**
     * @var Data
     */
    protected $helperData;

    /**
     * Configuration constructor.
     *
     * @param Cart $cart
     * @param Data $helperData
     */
    public function __construct(
        Cart $cart,
        Data $helperData
    ) {
        $this->cart = $cart;
        $this->helperData = $helperData;
    }

    /**
     * @param CoreConfiguration $subject
     * @param $result
     * @param ItemInterface $item
     * @param Product $selectionProduct
     *
     * @return mixed
     * @SuppressWarnings(Unused)
     */
    public function afterGetSelectionFinalPrice(
        CoreConfiguration $subject,
        $result,
        ItemInterface $item,
        Product $selectionProduct
    ) {
        if (!$this->helperData->isEnabled()) {
            return $result;
        }

        $items = $this->cart->getQuote()->getAllItems();

        foreach ($items as $childItem) {
            if ($childItem->getOriginalCustomPrice() && $selectionProduct->getId() === $childItem->getProductId()) {
                return $childItem->getOriginalCustomPrice();
            }
        }

        return $result;
    }
}
