<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Plugin\Model\Export;

use DateTime;
use DateTimeZone;
use Exception;
use Magento\Framework\Api\Search\DocumentInterface;
use Magento\Framework\Locale\ResolverInterface;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Ui\Component\MassAction\Filter;
use Magento\Ui\Model\Export\MetadataProvider as ExportMetadataProvider;
use Mageplaza\NameYourPrice\Helper\Data;

/**
 * Class MetadataProvider
 * @package Mageplaza\NameYourPrice\Plugin\Model\Export
 */
class MetadataProvider extends ExportMetadataProvider
{
    /**
     * @var Data
     */
    protected $helperData;

    /**
     * MetadataProvider constructor.
     *
     * @param Filter $filter
     * @param TimezoneInterface $localeDate
     * @param ResolverInterface $localeResolver
     * @param Data $helperData
     * @param string $dateFormat
     * @param array $data
     */
    public function __construct(
        Filter $filter,
        TimezoneInterface $localeDate,
        ResolverInterface $localeResolver,
        Data $helperData,
        $dateFormat = 'M j, Y h:i:s A',
        array $data = []
    ) {
        $this->helperData = $helperData;

        parent::__construct($filter, $localeDate, $localeResolver, $dateFormat, $data);
    }

    /**
     * @param ExportMetadataProvider $metadataProvider
     * @param callable $proceed
     * @param DocumentInterface $document
     * @param array $fields
     * @param array $options
     *
     * @return array
     * @throws Exception
     */
    public function aroundGetRowData(
        ExportMetadataProvider $metadataProvider,
        callable $proceed,
        DocumentInterface $document,
        $fields,
        $options
    ) {
        if (!$this->helperData->isEnabled()) {
            return $proceed($document, $fields, $options);
        }

        $row = [];
        foreach ($fields as $column) {
            $key = $document->getCustomAttribute($column)->getValue();
            if ($column === 'submitted_date') {
                $convertedDate = $this->localeDate->date(
                    new DateTime($key, new DateTimeZone('UTC')),
                    $this->locale,
                    true
                );
                $key           = $convertedDate->format('Y-m-d H:i:s');
            }
            if (is_array($key)) {
                $row[] = implode(',', $key);
            } else {
                $row[] = $key;
            }
        }

        return $row;
    }
}
