<?php

namespace MagePsycho\MassImporterPro\Block\Adminhtml\Import\Next\Edit;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Form extends \Magento\Backend\Block\Widget\Form\Generic
{
    /**
     * @var \MagePsycho\MassImporterPro\Model\System\Config\Source\EntityType
     */
    private $importEntityType;

    /**
     * @var \MagePsycho\MassImporterPro\Model\System\Config\Source\FileType
     */
    private $importFileType;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \MagePsycho\MassImporterPro\Model\System\Config\Source\EntityType $importEntityType,
        \MagePsycho\MassImporterPro\Model\System\Config\Source\FileType $importFileType,
        array $data = []
    ) {
        $this->importEntityType = $importEntityType;
        $this->importFileType = $importFileType;
        parent::__construct($context, $registry, $formFactory, $data);
    }

    public function getImport()
    {
        return $this->_coreRegistry->registry('current_import');
    }

    protected function _prepareForm()
    {
        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create(
            [
                'data' => [
                    'id' => 'edit_form',
                    'action' => $this->getUrl('*/*/new'),
                    'method' => 'get'
                ]
            ]
        );

        $fieldset = $form->addFieldset('base', ['legend' => __('Import Settings')]);

        $fieldset->addField(
            'entity_type',
            'select',
            [
                'name' => 'entity_type',
                'label' => __('Entity Type'),
                'title' => __('Entity Type'),
                'required' => true,
                'class' => 'validate-select',
                'style' => 'min-width: 245px',
                'values' => $this->importEntityType->getOptionsArray(),
            ]
        );
        $fieldset->addField(
            'file_type',
            'select',
            [
                'name' => 'file_type',
                'label' => __('File Type'),
                'title' => __('File Type'),
                'required' => true,
                'class' => 'validate-select',
                'style' => 'min-width: 245px',
                'values' => $this->importFileType->getOptionsArray(),
            ]
        );
        $fieldset = $form->addFieldset('csv_settings', ['legend' => __('CSV Settings')]);
        $fieldset->addField(
            'csv_settings_label',
            'text',
            [
                'name'         => 'csv_settings_label',
                'label'        => __("Currently Mass Importer Pro supports only the CSV file type, with comma (,) as delimiter and double quotes(\") as enclosure."),
            ]
        );
        $form->getElement('csv_settings_label')->setRenderer(
            $this->getLayout()->createBlock(
                \MagePsycho\MassImporterPro\Block\Adminhtml\Import\Edit\Renderer\Label::class
            )
        );

        $fieldset->addField(
            'csv_delimiter',
            'select',
            [
                'name'      => 'csv_delimiter',
                'label'     => __('Delimiter'),
                'required'  => true,
                'style' => 'min-width: 245px',
                'values'    => [',' => __('Comma(,)')],
                'value'     => ',',
                'disabled'  => true,
                'after_element_html' => '',
            ]
        );

        $enclosure = $fieldset->addField(
            'csv_enclosure',
            'select',
            [
                'name'      => 'csv_enclosure',
                'label'     => __('Enclosure'),
                'required'  => true,
                'style' => 'min-width: 245px',
                'values'    => ['"' => __('Double Quote(")')],
                'value'     => '"',
                'disabled'  => true,
                'after_element_html' => '',
            ]
        );

        $enclosure->setAfterElementHtml(
            "<script>
 require(
    ['jquery'],
    function($){
        $(function() {
            $('#csv_settings').hide();
            $(document).on('change', '#file_type', function() {
                var fileType = $('#file_type :selected').val();
                console.log('fileType::' + fileType);
                if (fileType == 'csv') {
                    $('#csv_settings').show();
                }
            });
        });
    }
 );
</script>"
        );

        $form->setUseContainer(true);
        $this->setForm($form);
        return parent::_prepareForm();
    }
}
