<?php

namespace MagePsycho\MassImporterPro\Controller\Adminhtml\History;

use Magento\Framework\App\Filesystem\DirectoryList;

use Magento\Backend\App\Action\Context;
use MagePsycho\MassImporterPro\Controller\Adminhtml\History as HistoryController;
use MagePsycho\MassImporterPro\Helper\Data as MipHelper;
use MagePsycho\MassImporterPro\Helper\Report as ReportHelper;
use MagePsycho\MassImporterPro\Model\HistoryFactory;
use Magento\Framework\Filesystem\Io\File;
use Magento\Ui\Component\MassAction\Filter;
use Magento\Framework\App\Response\Http\FileFactory;
use Magento\Framework\Controller\Result\RawFactory;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MassImporterPro
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Download extends HistoryController
{
    /**
     * @var \Magento\Framework\Controller\Result\RawFactory
     */
    protected $resultRawFactory;

    /**
     * @var \Magento\Framework\App\Response\Http\FileFactory
     */
    private $fileFactory;

    public function __construct(
        Context $context,
        MipHelper $mipHelper,
        ReportHelper $reportHelper,
        HistoryFactory $historyFactory,
        File $fileIo,
        Filter $filter,
        FileFactory $fileFactory,
        RawFactory $resultRawFactory
    ) {
        $this->fileFactory = $fileFactory;
        $this->resultRawFactory = $resultRawFactory;
        parent::__construct($context, $mipHelper, $reportHelper, $historyFactory, $fileIo, $filter);
    }

    public function execute()
    {
        $fileInfo = $this->fileIo->getPathInfo(
            $this->getRequest()->getParam('filename')
        );
        $fileName = $fileInfo['basename'];

        if (!$this->reportHelper->importFileExists($fileName)) {
            /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
            $resultRedirect = $this->resultRedirectFactory->create();
            $resultRedirect->setPath('*/import/index');
            return $resultRedirect;
        }

        $this->fileFactory->create(
            $fileName,
            null,
            DirectoryList::VAR_DIR,
            'application/octet-stream',
            $this->reportHelper->getReportSize($fileName)
        );

        /** @var \Magento\Framework\Controller\Result\Raw $resultRaw */
        $resultRaw = $this->resultRawFactory->create();
        $resultRaw->setContents($this->reportHelper->getReportOutput($fileName));
        return $resultRaw;
    }
}
