<?php

namespace MagePsycho\MipPricing\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Config implements ConfigInterface
{
    private const XML_PATH_ENABLED                      = 'magepsycho_massimporterpro/pricing/enabled';
    private const XML_PATH_DEBUG                        = 'magepsycho_massimporterpro/pricing/debug';

    private const XML_PATH_IDENTIFIER                   = 'magepsycho_massimporterpro/pricing/identifier';
    private const XML_PATH_RELATIVE_PRICE_ATTRIBUTE     = 'magepsycho_massimporterpro/pricing/relative_price_attribute';
    private const XML_PATH_ROUNDING_TYPE                = 'magepsycho_massimporterpro/pricing/rounding_type';

    private const XML_PATH_ROUND_TO_NEAREST_VALUE       = 'magepsycho_massimporterpro/pricing/round_to_nearest_value';
    private const XML_PATH_REINDEX_AFTER_IMPORT         = 'magepsycho_massimporterpro/pricing/reindex_after_import';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    public function __construct(
        ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @inheritDoc
     */
    public function getConfigFlag($xmlPath, $storeId = null)
    {
        return $this->scopeConfig->isSetFlag(
            $xmlPath,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @inheritDoc
     */
    public function getConfigValue($xmlPath, $storeId = null)
    {
        return $this->scopeConfig->getValue(
            $xmlPath,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    public function isEnabled($storeId = null)
    {
        return $this->getConfigFlag(self::XML_PATH_ENABLED, $storeId);
    }

    public function isActive($storeId = null)
    {
        return $this->isEnabled($storeId);
    }

    public function getDebugStatus($storeId = null)
    {
        return $this->getConfigValue(self::XML_PATH_DEBUG, $storeId);
    }

    public function getProductIdentifier($storeId = null)
    {
        return $this->getConfigValue(self::XML_PATH_IDENTIFIER, $storeId);
    }

    public function getRelativePriceAttribute($storeId = null)
    {
        return $this->getConfigValue(self::XML_PATH_RELATIVE_PRICE_ATTRIBUTE, $storeId);
    }

    public function getRoundingType($storeId = null)
    {
        return $this->getConfigValue(self::XML_PATH_ROUNDING_TYPE, $storeId);
    }

    public function getRoundingToNearestValue($storeId = null)
    {
        return $this->getConfigValue(self::XML_PATH_ROUND_TO_NEAREST_VALUE, $storeId);
    }

    public function getReindexAfterImport($storeId = null)
    {
        return $this->getConfigValue(self::XML_PATH_REINDEX_AFTER_IMPORT, $storeId);
    }
}
