<?php

namespace MagePsycho\MipPricing\Model\Import\Pricing\Validator;

use MagePsycho\MipPricing\Model\Import\Pricing\RowValidatorInterface;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class GroupPrice extends AbstractPrice implements RowValidatorInterface
{
    const PRICE_PREFIX           = 'group_price';

    /**
     * @var \MagePsycho\MipPricing\Model\Import\Pricing\StoreResolver
     */
    protected $storeResolver;

    public function __construct(
        \Magento\Customer\Api\GroupRepositoryInterface $groupRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \MagePsycho\MipPricing\Model\Import\Pricing\StoreResolver $storeResolver
    ) {
        $this->storeResolver = $storeResolver;
        parent::__construct($groupRepository, $searchCriteriaBuilder);
    }

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function isValid($value)
    {
        $this->_clearMessages();

        $groupPriceGroups = array_merge([self::VALUE_ALL], $this->customerGroups);
        foreach ($groupPriceGroups as $priceGroup) {
            if (isset($value[self::PRICE_PREFIX . ':' . $priceGroup])) {
                $price = $value[self::PRICE_PREFIX . ':' . $priceGroup];
                if (!strlen($price)) {
                    continue;
                }

                if (!$this->assertPriceFormat($price)) {
                    $this->_addMessages([self::ERROR_GROUP_DATA_INVALID]);
                    return false;
                }
            }
        }

        return true;
    }
}
