<?php

namespace MagePsycho\MipPricing\Model\System\Config\Source;

/**
 * @category   MagePsycho
 * @package    MagePsycho_MipPricing
 * @author     Raj KB <magepsycho@gmail.com>
 * @website    https://www.magepsycho.com
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class TierPriceImportType implements \Magento\Framework\Option\ArrayInterface
{
    const IMPORT_TYPE_MERGE             = 'merge';
    const IMPORT_TYPE_REPLACE_GROUP     = 'replace_group';
    const IMPORT_TYPE_REPLACE_ALL       = 'replace_all';

    protected $_options;

    public function getAllOptions($withEmpty = false)
    {
        if ($this->_options === null) {
            $this->_options = [
                [
                    'value' => self::IMPORT_TYPE_MERGE,
                    'label' => __('Merge'),
                ],
                [
                    'value' => self::IMPORT_TYPE_REPLACE_GROUP,
                    'label' => __('Replace (Group)'),
                ],
                [
                    'value' => self::IMPORT_TYPE_REPLACE_ALL,
                    'label' => __('Replace (All)'),
                ],
            ];
        }
        $options = $this->_options;
        if ($withEmpty) {
            array_unshift($options, ['value' => '', 'label' => '']);
        }
        return $options;
    }

    public function getOptionsArray($withEmpty = true)
    {
        $options = [];
        foreach ($this->getAllOptions($withEmpty) as $option) {
            $options[$option['value']] = $option['label'];
        }
        return $options;
    }

    public function getOptionText($value)
    {
        $options = $this->getAllOptions(false);
        foreach ($options as $item) {
            if ($item['value'] == $value) {
                return $item['label'];
            }
        }
        return false;
    }

    public function toOptionArray()
    {
        return $this->getAllOptions();
    }

    public function toOptionHash($withEmpty = true)
    {
        return $this->getOptionsArray($withEmpty);
    }
}
