<?php
/**
 * Copyright © MageWorx. All rights reserved.
 * See LICENSE.txt for license details.
 */
declare(strict_types=1);

namespace MageWorx\OptionInventory\Observer;


use Magento\Catalog\Model\Locator\LocatorInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\ObjectManagerInterface;
use MageWorx\OptionInventory\Helper\Stock as HelperStock;
use MageWorx\OptionInventory\Model\ResourceModel\Product\CollectionUpdaterStock;
use MageWorx\OptionBase\Helper\Data as BaseHelper;


class LinkedQtySourceUpdate implements ObserverInterface
{
    /**
     * Object Manager instance
     *
     * @var ObjectManagerInterface
     */
    protected $objectManager = null;

    /**
     * @var CollectionUpdaterStock
     */
    protected $collectionUpdaterStock;

    /**
     * @var BaseHelper
     */
    protected $baseHelper;

    /**
     * @var HelperStock
     */
    protected $helperStock;

    /**
     * LinkedQtySourceUpdate constructor.
     *
     * @param ObjectManagerInterface $objectManager
     * @param CollectionUpdaterStock $collectionUpdaterStock
     * @param BaseHelper $baseHelper
     * @param HelperStock $helperData
     */
    public function __construct(
        ObjectManagerInterface $objectManager,
        CollectionUpdaterStock $collectionUpdaterStock,
        BaseHelper $baseHelper,
        HelperStock $helperStock
    ) {
        $this->objectManager          = $objectManager;
        $this->collectionUpdaterStock = $collectionUpdaterStock;
        $this->baseHelper             = $baseHelper;
        $this->helperStock            = $helperStock;
    }

    /**
     * link salable qty - temporary solution
     *
     * @param Observer $observer
     */
    public function execute(Observer $observer): void
    {
        if (!$this->helperStock->validateLinkedQtyField()) {
            return;
        }

        $values = $observer->getDataToUpdate();
        foreach ($values as $value) {
            if ($value->getSkuIsValid()) {
                $optionId     = $value->getOptionId();
                $option       = $observer->getOptionCollection()->getItemById($optionId);
                $defaultStock = $this->baseHelper->updateValueQtyToSalableQty($value->getSku());
                $value->setQty($defaultStock);

                if ($option) {
                    $option->addValue($value);
                    $value->setOption($option);
                }
            }
        }
    }
}