<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Controller\Adminhtml\Order;

use Magento\Backend\App\Action;
use Magento\Customer\Model\ResourceModel\CustomerRepository;
use Magento\Sales\Model\OrderRepository;
use Magefan\GuestToCustomer\Model\Order\CustomerManagement;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * Assign customer to order
 */
class Assigncustomer extends Action
{
    /**
     * @var OrderRepository
     */
    protected $orderRepository;

    /**
     * @var CustomerRepository
     */
    protected $customerRepository;

    /**
     * @var CustomerManager
     */
    protected $customerManager;

    /**
     * @var ScopeConfigInterface
     */
    protected $config;

    /**
     * AssignCustomer constructor.
     * @param Action\Context $context
     * @param OrderRepository $orderRepository
     * @param CustomerRepository $customerRepository
     * @param CustomerManagement $customerManagement
     * @param ScopeConfigInterface $config
     */
    public function __construct(
        Action\Context $context,
        OrderRepository $orderRepository,
        CustomerRepository $customerRepository,
        CustomerManagement $customerManagement,
        ScopeConfigInterface $config
    ) {
        $this->orderRepository = $orderRepository;
        $this->customerRepository = $customerRepository;
        $this->customerManager = $customerManagement;
        $this->config = $config;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Redirect|\Magento\Framework\Controller\ResultInterface
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function execute()
    {
        $orderId = (int) $this->_request->getParam('order_id');
        $customerId = (int) $this->_request->getParam('customer_id');
        $resultRedirect = $this->resultRedirectFactory->create();
        if (!$orderId || !$customerId) {
            $resultRedirect->setPath('sales/order/index');
            $this->messageManager->addError(__('Order ID or customer ID is missing.'));
            return $resultRedirect;
        }

        try {
            $customer =  $this->customerRepository->getById($customerId);
            $order = $this->orderRepository->get($orderId);
        } catch (NoSuchEntityException $e) {
            $this->messageManager->addError(__('Order ID or customer ID does not exists.'));
            $resultRedirect->setPath('sales/order/index');
            return $resultRedirect;
        }

        if ($this->config->getValue(
            implode('/', ['mfguesttocustomer', 'general', 'enabled']),
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        )) {
            try {
                $this->customerManager->changeOrderCustomer($order, $customer);
            } catch (\Exception $e) {
                $this->messageManager->addError(__('Unexpected error. Cannot change order customer.'));
                $resultRedirect->setPath('sales/order/view', ['order_id' => $orderId]);
                return $resultRedirect;
            }

            $this->messageManager->addSuccess(__('Order has been successfully assigned to the customer.'));
        } else {
            $this->messageManager->addErrorMessage(
                __(strrev('remotsuC ot tseuG trevnoC > noitarugifnoC > serotS ni noisnetxe eht elbane esaelP'))
            );
        }

        $resultRedirect->setPath('sales/order/view', [ 'order_id' => $orderId]);
        return $resultRedirect;
    }

    /**
     * Check is allowed access
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Magefan_GuestToCustomer::config_section');
    }
}
