<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_LayeredNavigation
 * @copyright   Copyright (c) 2017 Mageplaza (http://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\LayeredNavigation\Model\Layer\Filter;

use Magento\CatalogSearch\Model\Layer\Filter\Attribute as AbstractFilter;
use Mageplaza\LayeredNavigation\Helper\Data as LayerHelper;

/**
 * Class Attribute
 * @package Mageplaza\LayeredNavigation\Model\Layer\Filter
 */
class Attribute extends AbstractFilter
{
	/** @var \Mageplaza\LayeredNavigation\Helper\Data */
	protected $_moduleHelper;

	/** @var bool Is Filterable Flag */
	protected $_isFilter = true;

	/** @var \Magento\Framework\Filter\StripTags */
	private $tagFilter;

	/**
	 * @param \Magento\Catalog\Model\Layer\Filter\ItemFactory $filterItemFactory
	 * @param \Magento\Store\Model\StoreManagerInterface $storeManager
	 * @param \Magento\Catalog\Model\Layer $layer
	 * @param \Magento\Catalog\Model\Layer\Filter\Item\DataBuilder $itemDataBuilder
	 * @param \Magento\Framework\Filter\StripTags $tagFilter
	 * @param \Mageplaza\LayeredNavigation\Helper\Data $moduleHelper
	 * @param array $data
	 */
	public function __construct(
		\Magento\Catalog\Model\Layer\Filter\ItemFactory $filterItemFactory,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Catalog\Model\Layer $layer,
		\Magento\Catalog\Model\Layer\Filter\Item\DataBuilder $itemDataBuilder,
		\Magento\Framework\Filter\StripTags $tagFilter,
		LayerHelper $moduleHelper,
		array $data = []
	)
	{
		parent::__construct(
			$filterItemFactory,
			$storeManager,
			$layer,
			$itemDataBuilder,
			$tagFilter,
			$data
		);
		$this->tagFilter     = $tagFilter;
		$this->_moduleHelper = $moduleHelper;
	}

	/**
	 * @inheritdoc
	 */
	public function apply(\Magento\Framework\App\RequestInterface $request)
	{
		if (!$this->_moduleHelper->isEnabled()) {
			return parent::apply($request);
		}

		$attributeValue = $request->getParam($this->_requestVar);
		if (empty($attributeValue)) {
			$this->_isFilter = false;

			return $this;
		}

		$attributeValue = explode(',', $attributeValue);

		$attribute = $this->getAttributeModel();
		/** @var \Magento\CatalogSearch\Model\ResourceModel\Fulltext\Collection $productCollection */
		$productCollection = $this->getLayer()
			->getProductCollection();
		if (count($attributeValue) > 1) {
			$productCollection->addFieldToFilter($attribute->getAttributeCode(), ['in' => $attributeValue]);
		} else {
			$productCollection->addFieldToFilter($attribute->getAttributeCode(), $attributeValue[0]);
		}

		$state = $this->getLayer()->getState();
		foreach ($attributeValue as $value) {
			$label = $this->getOptionText($value);
			$state->addFilter($this->_createItem($label, $value));
		}

		return $this;
	}

	/**
	 * @inheritdoc
	 */
	protected function _getItemsData()
	{
		if (!$this->_moduleHelper->isEnabled()) {
			return parent::_getItemsData();
		}

		$attribute = $this->getAttributeModel();

		/** @var \Mageplaza\LayeredNavigation\Model\ResourceModel\Fulltext\Collection $productCollection */
		$productCollection = $this->getLayer()->getProductCollection();

		if ($this->_isFilter) {
			$productCollection = $productCollection->getCollectionClone()
				->removeAttributeSearch($attribute->getAttributeCode());
		}

		$optionsFacetedData = $productCollection->getFacetedData($attribute->getAttributeCode());

		if (count($optionsFacetedData) === 0
			&& $this->getAttributeIsFilterable($attribute) !== static::ATTRIBUTE_OPTIONS_ONLY_WITH_RESULTS
		) {
			return $this->itemDataBuilder->build();
		}

		$productSize = $productCollection->getSize();

		$itemData   = [];
		$checkCount = false;

		$options = $attribute->getFrontend()
			->getSelectOptions();
		foreach ($options as $option) {
			if (empty($option['value'])) {
				continue;
			}

			$value = $option['value'];

			$count = isset($optionsFacetedData[$value]['count'])
				? (int)$optionsFacetedData[$value]['count']
				: 0;

			// Check filter type
			if ($this->getAttributeIsFilterable($attribute) == static::ATTRIBUTE_OPTIONS_ONLY_WITH_RESULTS
				&& (!$this->_moduleHelper->getFilterModel()->isOptionReducesResults($this, $count, $productSize))
			) {
				continue;
			}

			if ($count > 0) {
				$checkCount = true;
			}

			$itemData[] = [
				'label' => $this->tagFilter->filter($option['label']),
				'value' => $value,
				'count' => $count
			];
		}

		if ($checkCount) {
			foreach ($itemData as $item) {
				$this->itemDataBuilder->addItemData($item['label'], $item['value'], $item['count']);
			}
		}

		return $this->itemDataBuilder->build();
	}
}
